#include "stdafx.h"



//audio/mpeg
const TCHAR* commentStr=
_T("<p>I changed back to original design because MySQL is not really trustworthy for thread-safe\
It is like window's service's 'appartment threading mode. Each thread running in its own space. \
The most siganificent improvement is the DB connection recovery. However, it is not fully tested.\
I hope visitors can help me test this. </p>");

const TCHAR* errorMessage = 
_T("<p> cannot find data in database</p>");

const TCHAR* MediaTypeLongName[2]=
{
	_T("image"),
	_T("music")
};

const TCHAR* MediaTypeShortName[2]=
{
	_T("img"),
	_T("embed")
};

ULONG result;


#define SAFE_CALL(X) if ((result=X)!=NO_ERROR) printf("error %u\n", result);


HttpServerWorker::HttpServerWorker()
{
	int i;
	hReqQueue = NULL;
	threadHandle       = NULL;
	semaphore          = NULL;
	canStop            = true;
	isConnected        = false;
	isAssigned         = false;
	connectionId       = 0;
	
	requestBuffer =NULL; 
	requestBufferSize  = 0;
	stream             = NULL;
	

	semaphore   = NULL;

	for (i = 0; i< MaxMediaTypeCount; i ++)
	{  
		currentMediaId[i] = 1;
		currentMediaPage[i] =  0;
		myMediaHashTable[i].mediaType=i;
	}

}

   

bool HttpServerWorker::recoverFromDB()
{
	int i, j;
	_ftprintf(stream, _T("attempt to recover from DB error\n"));
	for (i = 0; i < MaxMediaTypeCount; i++)
	{
		for (j = 0; j < CacheTableSize; j ++)
		{
			if (myMediaHashTable[i].sqlResTable[j] != NULL)
			{
				freeResult(myMediaHashTable[i].sqlResTable[j]);
			}
			myMediaHashTable[i].sqlResTable[j] = NULL;
		}
	}
	if (!MySqlHelper::getHostIp(myDatabaseIP))
	{
		return false;
	}
	if (!connectDB(&mysql, myDatabaseIP))
	{
		_ftprintf(stream, _T("attempt to recover from DB failed\n"));
		return false;
	}
	_ftprintf(stream, _T("attempt to recover from DB succeeded\n"));
	return true;
}

void HttpServerWorker::analysis(int mediaType)
{	
	char buffer[64];
	time_t ltime;
	if (mediaType == -1)
	{
		return;
	}
	retrieveAddress(requestBuffer->Address.pRemoteAddress, buffer);

	time(&ltime);
	sprintf(buffer + strlen(buffer), " [%d] [%s", currentMediaId[mediaType], ctime(&ltime));
	buffer[strlen(buffer)-1]=']';
 	fprintf(stream, "%s\n",  buffer);
	fflush(stream);
	printf("%s\n", 	buffer);
}


	 
bool HttpServerWorker::init(LPSTR dbAddress)
{
	//int len;
	bool hasError = false;
	LPBYTE tempPtr = NULL;
	/*
	len = _tcsclen(dbAddress);
	
	for (int i = 0; i < len; i ++)
	{
		myDatabaseIP[i] = dbAddress[i];
	}
	*/

	//_tcscpy(myDatabaseIP, dbAddress);
	if (!hasError)
	{
		semaphore=CreateSemaphore(NULL, 0, MaxPossibleResponseFromSingleConnection,	NULL);
		hasError = (semaphore == NULL);
	}
	
	//InitializeCriticalSection(&cs);
	
	if (!hasError)
	{
		hasError = !connectDB(&mysql, dbAddress);
	}

	if (!hasError)
	{
		isConnected = true;
		requestBuffer = (PHTTP_REQUEST)(new char[DefaultRequestBufferSize]);
		hasError = 	(requestBuffer == NULL);   
	}
	if (!hasError)
	{
		hasError = ! mySqlHelper.init(&mysql);
	}

	requestBufferSize = DefaultRequestBufferSize;
	return !hasError;
}

void HttpServerWorker::uninit()
{
	int i, j;
	delete[] requestBuffer;
	CloseHandle(semaphore);
	for (i = 0; i < MaxMediaTypeCount; i++)
	{
		for (j = 0; j < CacheTableSize; j ++)
		{
			if (myMediaHashTable[i].sqlResTable[j] != NULL)
			{
				freeResult(myMediaHashTable[i].sqlResTable[j]);
			}  			
		}
	}

	if (isConnected)
	{
		disconnectDB(&mysql); 
	}	 
}


int HttpServerWorker::hash(MyHashTable& hashTable, int id, bool isThumb)
{
	int index=id%CacheTableSize;  	
	if (id!=hashTable.cacheTable[index])
	{
		if (hashTable.cacheTable[index] != -1)
		{
			freeResult(hashTable.sqlResTable[index]);
		}
		hashTable.sqlResTable[index]=selectMediaDataByIndex(hashTable.mediaType, id, 
			hashTable.addressTable[index], hashTable.sizeTable[index], isThumb);
		if (hashTable.sqlResTable[index]==NULL)
		{
			hashTable.cacheTable[index]=-1;
			
			return -1;
		}
		hashTable.cacheTable[index]=id;		
	}	  	
	return index;
}


MyHashTable::MyHashTable()
{
	for (int i=0; i<CacheTableSize; i++)
	{
		cacheTable[i]=-1;
		addressTable[i]=NULL;
		sizeTable[i]=-1;
		sqlResTable[i]=NULL;
	}
}


void HttpServerWorker::errHandle(const _TCHAR* reason)
{
	_tprintf(_T("%s error %d\n"), reason, mysql_error(&mysql));
}


void HttpServerWorker::createEntryPage(HTTP_DATA_CHUNK& dataChunk)
{
	int len;
	//_stprintf(htmlBuffer, _T("%s <a href=\"image1\"> picture </a> &nbsp &nbsp &nbsp <a href=\"music1\">  music </a>"), textBegin);
	_stprintf(htmlBuffer, _T("%s <a href=\"image\"> picture </a> "), textBegin);
	len=_tcslen(htmlBuffer);
	_stprintf(htmlBuffer+len, _T("%s"), commentStr);
	len=_tcslen(htmlBuffer);
	_stprintf(htmlBuffer+len, _T("%s"), textEnd);	

	dataChunk.DataChunkType=HttpDataChunkFromMemory;
	dataChunk.FromMemory.pBuffer=htmlBuffer;
	dataChunk.FromMemory.BufferLength=_tcslen(htmlBuffer)*sizeof(TCHAR);	
}

void HttpServerWorker::createPage(int mediaType, HTTP_DATA_CHUNK& dataChunk, bool refresh)
{
	int i, len, batchNumber, currentId, currentPageNumber, index;
  
	len = 0;
	batchNumber       = MediaBatchNumber;
	currentId         = currentMediaId[mediaType];
	currentPageNumber = currentMediaPage[mediaType];

	_stprintf(htmlBuffer+len, textHeader);
	len = _tcslen(htmlBuffer); 

 	_stprintf(htmlBuffer+len, _T("<form name=myform method=get action=http://192.168.2.11/%s%d>"),
		 MediaTypeLongName[mediaType], currentId);
	len = _tcslen(htmlBuffer); 

	_stprintf(htmlBuffer+len, _T("<table width=100% > <tr><td> <a href=\"%sprev%d\"> prev  </a></td>"),
		MediaTypeLongName[mediaType], __max(0, currentMediaPage[mediaType]-1));
	len=_tcslen(htmlBuffer);
	for (i = 1; i <= batchNumber; i++)
	{
		index = i+currentPageNumber*batchNumber;
		if (index > maxMediaId[mediaType] - minMediaId[mediaType] + 1)
		{
			break;
		}
		_stprintf(htmlBuffer+len, _T("<td><a href=\"%s%d\"> <img src=\"imagedatathumb%d\"> </a></td>"), 
			MediaTypeLongName[mediaType],			
			i+currentPageNumber*batchNumber, index);
		len=_tcslen(htmlBuffer);
	}
	_stprintf(htmlBuffer+len, _T("<td><a href=\"%snext%d\"> next </a></td></tr></table></form>"),
		MediaTypeLongName[mediaType], currentMediaPage[mediaType] + 1);

	len=_tcslen(htmlBuffer);
	_stprintf(htmlBuffer+len,_T("<table border=\"0\" width=\"100%\" cellpadding=\"0\"><td width=600 valign=top>"));

	len=_tcslen(htmlBuffer);

	_stprintf(htmlBuffer+len, _T("<%s  src=%sdata%d></td>"), MediaTypeShortName[mediaType],
		MediaTypeLongName[mediaType], currentId);	

	len=_tcslen(htmlBuffer);  	
	_stprintf(htmlBuffer + len, _T("<td width=10 valign=top><form name=hideForm method=get><input type=hidden name=hide value=%d>"), currentId);
	len=_tcslen(htmlBuffer);
	_stprintf(htmlBuffer + len, _T("<input type=button width=10 value=hide\
		  onclick = document.getElementById(\"hideform\").submit()></form></td>"));


	len=_tcslen(htmlBuffer);
	_stprintf(htmlBuffer + len, _T("<td width=10	valign=top><form name=showForm method=get><input type=hidden name=show value=%d>"), currentId);
	len=_tcslen(htmlBuffer);
	_stprintf(htmlBuffer + len, _T("<input type=button width=10 value=show\
		  onclick = document.getElementById(\"showform\").submit()></form></td>"));

	len=_tcslen(htmlBuffer);
	_stprintf(htmlBuffer + len, _T("<td width=10 valign=top><form name=\"uploadform\" method=post enctype=\"multipart/form-data\">\
		<input type=\"file\" name=\"imgfile\" size=10 onchange=\"viewmypic(showimg,this.form.imgfile);\"/>\
		<input type=\"submit\" value=\"uploadfile\" width=10></form>\
		<td width=10 valign=top><img name=\"showimg\" id=\"showimg\" src=\"\" style=\"display:none;\"/></table>"));
		  //onclick=document.getElementById(\"myForm\").submit()>"));
	len=_tcslen(htmlBuffer);

	_stprintf(htmlBuffer +len, _T("%s"), textBegin);
	

	len=_tcslen(htmlBuffer);
	_stprintf(htmlBuffer+len, _T("%s"), textEnd);

	//_tprintf(_T("the html buffer is\n\n%s\n\n"), htmlBuffer);
	dataChunk.DataChunkType=HttpDataChunkFromMemory;
	dataChunk.FromMemory.pBuffer=htmlBuffer;
	dataChunk.FromMemory.BufferLength=_tcslen(htmlBuffer)*sizeof(TCHAR);
}

bool HttpServerWorker::checkHideCounter(int& counter, int mediaType, int id)
{
	switch (mediaType)
	{
	case 0:
		return selectNumberData(counter, id, "picture_hide_counter", "mypicture");
		break;
	default:
		return false;
	}
}


bool HttpServerWorker::getMediaData(int mediaType, HTTP_DATA_CHUNK& dataChunk, bool isThumb)
{
	int index = 0;	
	dataChunk.DataChunkType=HttpDataChunkFromMemory;
	int hideCounter = 0;
	if (isThumb)
	{
		index=hash(myThumbHashTable[mediaType], currentMediaId[mediaType] + 
			minMediaId[mediaType] - 1, isThumb);	
	}
	else
	{
		 if (!checkHideCounter(hideCounter, mediaType, currentMediaId[mediaType] + 
			minMediaId[mediaType] - 1))
		 {
			 return false;
		 }
		 if (hideCounter <= 0)
		 {
			 index=hash(myMediaHashTable[mediaType], currentMediaId[mediaType] + 
				minMediaId[mediaType] - 1, isThumb);	
		 }
	}
	if (!isThumb && hideCounter <= 0 && index == -1 || isThumb&&index == -1)
	{
		_ftprintf(stream, _T("get media data of %d failed\n"), currentMediaId[mediaType]);
		//should create error data page!!!
		//dataChunk.FromMemory.pBuffer = (void*)errorMessage;
		//dataChunk.FromMemory.BufferLength = _tcslen(errorMessage)*sizeof(TCHAR);
		return false;
	}
	if (isThumb)
	{
		dataChunk.FromMemory.pBuffer      = myThumbHashTable[mediaType].addressTable[index];
		dataChunk.FromMemory.BufferLength = myThumbHashTable[mediaType].sizeTable[index];
	}
	else
	{
		if (hideCounter <= 0)
		{
			dataChunk.FromMemory.pBuffer      = myMediaHashTable[mediaType].addressTable[index];
			dataChunk.FromMemory.BufferLength = myMediaHashTable[mediaType].sizeTable[index];
		}
		else
		{
			dataChunk.FromMemory.pBuffer      = hideImagePtr;
			dataChunk.FromMemory.BufferLength = hideImageSize;
		}
	}

	return true;
}

bool HttpServerWorker::increaseInt(int id, int data, char* field, char* table)
{
	char buffer[256];
	sprintf(buffer, "update %s set %s = %s+%d where id = %d", table, field, field, data, id);

	if (mysql_real_query(&mysql, buffer, strlen(buffer)))
	{
		_ftprintf(stream, _T("query error %s\n"), mysql_error(&mysql));	
		return false;
	}
	return true;
}

bool HttpServerWorker::decreaseInt(int id, unsigned int data, char* field, char* table)
{
	char buffer[256];
	sprintf(buffer, "update %s set %s = %s-%d where id = %d", table, field, field, data, id);

	if (mysql_real_query(&mysql, buffer, strlen(buffer)))
	{
		_ftprintf(stream, _T("query error %s\n"), mysql_error(&mysql));	
		return false;
	}
	return true;
}


bool HttpServerWorker::doSendQuery(int mediaType, HTTP_RESPONSE& response, 
									 HTTP_DATA_CHUNK& dataChunk, PCWSTR action)
{		
	int id = 0;
	//wprintf(L"%s\n", action);
	if (swscanf(action, L"?hide=%d", &id) == 1)
	{  		
		id = RangeResult(id, 1, maxMediaId[mediaType]);
		currentMediaId[mediaType] = id;
		switch (mediaType)
		{
		case 0:
			increaseInt(id, 1, "picture_hide_counter", "mypicture"); 
			currentMediaPage[mediaType] = (id - 1) / MediaBatchNumber;
			createPage(mediaType, dataChunk, true);	
			ADD_KNOWN_HEADER(response, HttpHeaderContentType, TextFormat);
			return true;   
			break;
		default:
			break;
		}		
	}
	else
	{
		if (swscanf(action, L"?show=%d", &id) == 1)
		{
			 id = RangeResult(id, 1, maxMediaId[mediaType]);
			currentMediaId[mediaType] = id;
			switch (mediaType)
			{
			case 0:
				decreaseInt(id, 1, "picture_hide_counter", "mypicture"); 
				currentMediaPage[mediaType] = (id - 1) / MediaBatchNumber;
				createPage(mediaType, dataChunk, true);	
				ADD_KNOWN_HEADER(response, HttpHeaderContentType, TextFormat);
				return true;   
				break;
			default:
				break;
			}	
			return true;
		}
	}
	return false;
}


bool HttpServerWorker::doSendHttpResponse(int mediaType, HTTP_RESPONSE& response, 
									 HTTP_DATA_CHUNK& dataChunk, const WCHAR* action)
{
	int id, pageId;

	if (swscanf(action, L"next%d", &pageId)==1)
	{			 
		currentMediaPage[mediaType] = RangeResult(pageId, 0, maxMediaPage[mediaType]-1);
	
		currentMediaId[mediaType] = 1 + currentMediaPage[mediaType] * MediaBatchNumber;		

		createPage(mediaType, dataChunk);
		ADD_KNOWN_HEADER(response, HttpHeaderContentType, TextFormat);
		return true;
	}

	if (swscanf(action, L"prev%d", &pageId)==1)
	{				 
		currentMediaPage[mediaType] = RangeResult(pageId, 0, 
			maxMediaPage[mediaType]);
		
		currentMediaId[mediaType] = 1 + currentMediaPage[mediaType] * MediaBatchNumber; 

		createPage(mediaType, dataChunk);
		ADD_KNOWN_HEADER(response, HttpHeaderContentType, TextFormat);
		return true;		
	}
			
	if (swscanf(action, L"data%d", &id)==1)
	{				
		switch (mediaType)
		{
		case 0:	 					
			ADD_KNOWN_HEADER(response, HttpHeaderContentType, "image/jpeg");
			//currentImageId=id;
			break;
		case 1:
			ADD_KNOWN_HEADER(response, HttpHeaderContentType, "audio/mpeg");
			//currentMusicId=id;
			break;
		}
		currentMediaId[mediaType]= RangeResult(id, 1, maxMediaId[mediaType]) ; 
		return getMediaData(mediaType, dataChunk, false);
	}	
	if (swscanf(action, L"datathumb%d", &id)==1) 
	{
		switch (mediaType)
		{
		case 0:	 					
			ADD_KNOWN_HEADER(response, HttpHeaderContentType, "image/jpeg");
			//currentImageId=id;
			break;
		case 1:
			ADD_KNOWN_HEADER(response, HttpHeaderContentType, "audio/mpeg");
			//currentMusicId=id;
			break;
		} 
		currentMediaId[mediaType]= RangeResult(id, 1, maxMediaId[mediaType]) ; 
		return getMediaData(mediaType, dataChunk, true);
	}
 
	if (swscanf(action, L"%d", &id)==1)
	{  
		id = RangeResult(id, 1, maxMediaId[mediaType]);
		currentMediaPage[mediaType] = (id - 1) / MediaBatchNumber;
		currentMediaId[mediaType] = currentMediaPage[mediaType] *
			MediaBatchNumber + (id - 1)% MediaBatchNumber + 1;
	
		createPage(mediaType, dataChunk);

		ADD_KNOWN_HEADER(response, HttpHeaderContentType, TextFormat);
		return true;
	}
	//this is the default unknow request			

	wprintf(L"unknown request[%s]\n", action);
	currentMediaPage[mediaType] = (rand()% 100 + 1) * 
		  (maxMediaPage[mediaType]  / 100);
	currentMediaId[mediaType] = currentMediaPage[mediaType] *
		MediaBatchNumber + 1;
	createPage(mediaType, dataChunk);
	ADD_KNOWN_HEADER(response, HttpHeaderContentType, TextFormat);		
	return true;	
}
 
//const unsigned int MaxBufferSize = 1024*1024;
//char buffer[MaxBufferSize];

void skipSpace(char*& ptr)
{
	while (*ptr==' '||*ptr=='\r'||*ptr=='\n')
	{
		ptr++;
	}
}

bool HttpServerWorker::sendHttpResponseHelper(int mediaType, HTTP_RESPONSE& response)
{
	int retValue = 0;
	DWORD bytesSent = 0;

	retValue = HttpSendHttpResponse(
                    hReqQueue,           // ReqQueueHandle
                    requestBuffer->RequestId, // Request ID                    
					0,
                    &response,           // HTTP response
                    NULL,                // pReserved1
                    &bytesSent,          // bytes sent   (OPTIONAL)
                    NULL,                // pReserved2   (must be NULL)
                    0,                   // Reserved3    (must be 0)
                    NULL,                // LPOVERLAPPED (OPTIONAL)
                    NULL                 // pReserved4   (must be NULL)
                    ); 

			
	_ftprintf(stream, _T("bytesSent %u, requestID=%I64u of connectionId %I64u \n"), bytesSent,
		requestBuffer->RequestId, requestBuffer->ConnectionId);
    if(retValue != NO_ERROR)
    {
        _ftprintf(stream, _T("HttpSendHttpResponse failed with %u \n"), retValue);
		return false;
	}
	else
	{  		
		analysis(mediaType); 
		return true;
	}    
}

//assume searching for 29 "-" plus certain number of random string.
int boundaryStringSearch(const char* src, const char* subStr, int srcLength, int subLength)
{
	//first let's search for 29 consecutive "-"
	int result = 0;
	int counter=0;
	
	//find first non "-" starting point
	while (src[result] == '-' && result < srcLength)
	{	
		for (counter =0; counter <29; counter++)
		{
			if (src[result+counter] != '-')
			{
				break;
			}
		}
		if (counter != 29)
		{
			result += counter;			
		}
		else
		{
			if (memcmp(src + result + 29, subStr, subLength) == 0)
			{
				return result;
			}
			else
			{
				result ++;
			}
		}
	}
	
	//so, we now know we start with non '-'
	while (result < srcLength)
	{
		//fast leap
		if (src[result] != '-' && src[result + 28] != '-')
		{
			result += 29;
		}
		else
		{
			if (src[result] != '-')
			{
				result ++;
			}
			else
			{
				for (counter = 0; counter < 29; counter++)
				{
					if (src[result + counter] != '-')
					{
						break;
					}
				}
				if (counter != 29)
				{
					result += counter;
				}
				else
				{
					if (memcmp(src + result + counter, subStr, subLength) == 0)
					{
						return result;
					}
					else
					{
						result ++;
					}
				}
			}
		}
	}
	//if we reach here, something is wrong.
	if (result >= srcLength)
	{
		return -1;
	}
}


int HttpServerWorker::doRecvFile()
{
    bool           result = true;
	DWORD          retValue = 0;            
  
	int mediaType = -1;	

	unsigned long int byteRecved =0;  
	unsigned long int accumRecvedSize =0;
	unsigned long int totalFileSize=0;
	char* buffer = NULL;
	char* curPtr = NULL, *ptr = NULL;
	int resultValue =0;
	bool isFirst = true;
	int nameLength = 0;
	int fileStartOffset = 0, fileEndOffset = 0;
	char* boundaryStr  = NULL;
	char* fileStartPtr = NULL;
	int srcLength = 0;
	char* srcPtr = NULL;
	int offset = 0;
	int repeatId =0;
	//const char* HeaderStr1="Content-Disposition: form-data; name=\"";
	//const char* HeaderStr2="\";
	//filename=\"%s\" Content-Type: %s/%s\r\n\r\n";

	TCHAR inputName[MAX_PATH];
	TCHAR fileName[MAX_PATH];

  	sscanf(requestBuffer->Headers.KnownHeaders[HttpHeaderContentLength].pRawValue, "%d", 
		&totalFileSize);
	if ((buffer = (char*)malloc(totalFileSize)) == NULL)
	{
		return result;
	}
	curPtr = buffer;

	do
	{
		result = true;
		resultValue = HttpReceiveRequestEntityBody(hReqQueue, requestBuffer->RequestId, 0,
			curPtr,  totalFileSize - accumRecvedSize, &byteRecved, NULL);
		switch (resultValue)
		{
		case NO_ERROR:
			result = true;
			break;
		default:
			result = false;
		}
		if (!result)
		{
			break;
		}  

		if (isFirst)
		{
			isFirst = false;
			ptr = curPtr;
			boundaryStr = strstr(requestBuffer->Headers.KnownHeaders[HttpHeaderContentType].pRawValue,
				"boundary=");
			boundaryStr += 9;//strlen("boundary=");
			fileStartPtr = strstr((char*)ptr, boundaryStr);
			fileStartPtr += strlen(boundaryStr);
			fileStartPtr = strstr(fileStartPtr, "name=\"");
			fileStartPtr += 6;
			ptr = strchr(fileStartPtr, '"');
			
			strncpy((char*)inputName, fileStartPtr , ptr - fileStartPtr);
			fileStartPtr = strstr(ptr, "filename=\"");
			fileStartPtr += 10;
			ptr = strchr(fileStartPtr, '"');
			nameLength = RangeResult(ptr - fileStartPtr, 0, 99);
		
			strncpy((char*)fileName, fileStartPtr , nameLength);
			fileName[(nameLength + 1)/sizeof(TCHAR)]='\0';			
			fileStartPtr = strchr(ptr, ':');  
			
			fileStartPtr = strstr(ptr, "\r\n\r\n");
			fileStartPtr += 4;
			//skipSpace(fileStartPtr);
			//resultValue = sscanf(fileStartPtr, HeaderStr, inputName);//, fileName, typeName, subTypeName);
			//resultValue = sscanf(fileStartPtr, HeaderStr, inputName, fileName, typeName, subTypeName);						
		}
			
		accumRecvedSize += byteRecved; 
		if (accumRecvedSize == totalFileSize)
		{
			break;
		}
		curPtr += 	byteRecved;
		//totalFileSize++;
	}
	while (accumRecvedSize < totalFileSize);
	if (result)
	{
		 
		if (curPtr == buffer)
		{	
			srcPtr = fileStartPtr;
			srcLength = byteRecved - (fileStartPtr - buffer); 			
		}
		else
		{
			srcPtr = curPtr;   			
			srcLength = byteRecved;
		} 
		offset = boundaryStringSearch(srcPtr, boundaryStr + 27, srcLength, strlen(boundaryStr+27));
	
		result = (offset >= 0);
		if (result)
		{
			fileEndOffset = srcLength - offset + 2; //because of CLRF
			fileStartOffset = fileStartPtr - buffer;			 	
			
			//saveJpgFile(_T("mytest.jpg"), (LPBYTE)fileStartPtr, totalFileSize - fileStartOffset - 
			//	fileEndOffset);
			repeatId = mySqlHelper.doInsertPicture(fileName, (LPBYTE)fileStartPtr, 
				totalFileSize - fileStartOffset - fileEndOffset);
			if (repeatId != 0)
			{
				result = false;
			}		
		}
		if (result)
		{
			result = updateMediaDataRange(0);
		}	
	}
	free(buffer);	

	if (result)
	{
		return 0;
	}
	else
	{
		return repeatId;
	}
}

bool HttpServerWorker::showUploadedMediaData(int mediaType, int repeatId)
{
	int id;	 
    DWORD           bytesSent;

	HTTP_RESPONSE   response; 
    HTTP_DATA_CHUNK dataChunk;

	INITIALIZE_HTTP_RESPONSE(&response, 200, "ok");
	memcpy(&response.Version, &requestBuffer->Version, sizeof(HTTP_VERSION));

	if (repeatId > 0)
	{
		id = repeatId;
	}
	else
	{
		id = maxMediaId[mediaType] - minMediaId[mediaType] + 1;
	}

	currentMediaPage[mediaType] = (id - 1) / MediaBatchNumber;
	currentMediaId[mediaType] = id;

	createPage(mediaType, dataChunk);

	ADD_KNOWN_HEADER(response, HttpHeaderContentType, TextFormat); 	

	response.EntityChunkCount         = 1;
	response.pEntityChunks            = &dataChunk;
	bytesSent=0;

	sendHttpResponseHelper(mediaType, response);
	return true;
}
 

bool HttpServerWorker::sendHttpResponse()
{
    HTTP_RESPONSE   response;
    HTTP_DATA_CHUNK dataChunk;
    bool           result = false;
	DWORD          retValue = 0;            
    DWORD           bytesSent;
	int repeatId =0;
	int mediaType = -1;
  
	if (_wcsnicmp(requestBuffer->CookedUrl.pAbsPath, L"/image", 6)==0)
	{
		mediaType = 0;
	}
	else
	{
		mediaType = 1;
	}

	if (requestBuffer->Verb == HttpVerbPOST)
	{
		repeatId = doRecvFile();	
		showUploadedMediaData(mediaType, repeatId); 
		return true;			
	}

    INITIALIZE_HTTP_RESPONSE(&response, 200, "ok");
	memcpy(&response.Version, &requestBuffer->Version, sizeof(HTTP_VERSION));

	if (requestBuffer->CookedUrl.pQueryString != NULL)
	{	   
		result= doSendQuery(mediaType, response, dataChunk, requestBuffer->CookedUrl.pQueryString);
	}
	else
	{  
		//_tprintf(_T("\n\nthe absPath=%s\n\n"), requestBuffer->CookedUrl.pAbsPath);
		if (requestBuffer->CookedUrl.pAbsPath==NULL)
		{
			createEntryPage(dataChunk);
			ADD_KNOWN_HEADER(response, HttpHeaderContentType, TextFormat);
		}
		else
		{
			if (_wcsnicmp(requestBuffer->CookedUrl.pAbsPath, L"/image", 6)==0)
			{
				if (!doSendHttpResponse(0, response, dataChunk, requestBuffer->CookedUrl.pAbsPath+6))
				{
					return false;
				}
				mediaType = 0;
			}
			else
			{
				if (_wcsnicmp(requestBuffer->CookedUrl.pAbsPath, L"/music", 6)==0)
				{
					if (!doSendHttpResponse(1, response, dataChunk, requestBuffer->CookedUrl.pAbsPath+6))
					{
						return false;
					}
					mediaType = 1;
				}
				else
				{
					createEntryPage(dataChunk);
					ADD_KNOWN_HEADER(response, HttpHeaderContentType, TextFormat);
				}
			}
		}
	} 

	response.EntityChunkCount         = 1;
    response.pEntityChunks            = &dataChunk;
	bytesSent=0;
	
    sendHttpResponseHelper(mediaType, response);  
	return true;
}

bool HttpServerWorker::selectNumberData(int& data, int id, char* field, char* table)
{
	char buffer[256];
	int tryCount = 0;
	const int MaxTryCount = 3;
	bool queryError = false;
	MYSQL_ROW row;
	MYSQL_RES* res = NULL;


	sprintf(buffer, "select %s from %s where id=%d;", field, table, id);

 	do
	{
		if (!queryError)
		{
			if (mysql_real_query(&mysql, buffer, strlen(buffer)))
			{
				//_ftprintf(stream, _T("query error %d\n"), mysql_error(&mysql));	
				fprintf(stream, "query error %s\n", mysql_error(&mysql));	
				queryError = true;
			}
		}

		if (!queryError)
		{
			if ((res=mysql_store_result(&mysql))==NULL)
			{
				//_ftprintf(stream, _T("store result error %d\n"), mysql_error(&mysql));
				fprintf(stream, "store result error %s\n", mysql_error(&mysql));
				queryError = true;
			}
		}
		if (!queryError)
		{  
			if ((row=mysql_fetch_row(res))==NULL)
			{
				//_ftprintf(stream, _T("row error %d\n"), mysql_error(&mysql));
				fprintf(stream, "row error %s\n", mysql_error(&mysql));
				//queryError = true;
				//may not be error, just cannot find data!!!!
				return false;//cannot find data!!
			}
		}
		
		if (queryError)
		{
			if (!recoverFromDB())
			{
				_ftprintf(stream, _T("%d attempts failed to recover from DB error connection\n"),
					++tryCount);	
				Sleep(1000);
			}
			else
			{
				queryError = false;
			}			
		} 
		else
		{
			break;
		}
	}
	while (tryCount < MaxTryCount) ;

	if (queryError)
	{
		isConnected = false;
		return false;
	}

	sscanf(row[0], "%d", &data);
	mysql_free_result(res);	
	return true;
}


void* HttpServerWorker::selectMediaDataByIndex(int mediaType, int id, char*& ptr, 
											   int& length, bool isThumb)
{
	char buffer[256];
	char* pDataField;
	int tryCount = 0;
	const int MaxTryCount = 3;
	bool queryError = false;
	MYSQL_ROW row;
	unsigned long * rowLengths;
	MYSQL_RES* res = NULL;
	switch (mediaType)
	{
	case 0:
		if (isThumb)
		{
			pDataField = "picture_thumb from mypicture";
		}
		else
		{
			pDataField = "picture_image from mypicture";
		}	 		
		break;
	case 1:
		pDataField = "music_data from mymusic";	 		
		break;
	}

	sprintf(buffer, "select %s where id=%d;", pDataField, id);


	do
	{
		if (!queryError)
		{
			if (mysql_real_query(&mysql, buffer, strlen(buffer)))
			{
				//_ftprintf(stream, _T("query error %d\n"), mysql_error(&mysql));	
				fprintf(stream, "query error %s\n", mysql_error(&mysql));	
				queryError = true;
			}
		}

		if (!queryError)
		{
			if ((res=mysql_store_result(&mysql))==NULL)
			{
				//_ftprintf(stream, _T("store result error %d\n"), mysql_error(&mysql));
				fprintf(stream, "store result error %s\n", mysql_error(&mysql));
				queryError = true;
			}
		}
		if (!queryError)
		{  
			if ((row=mysql_fetch_row(res))==NULL)
			{
				//_ftprintf(stream, _T("row error %d\n"), mysql_error(&mysql));
				fprintf(stream, "row error %s\n", mysql_error(&mysql));
				//queryError = true;
				//may not be error, just cannot find data!!!!
				return NULL;//cannot find data!!
			}
		}
		if (!queryError)
		{	
			if ((rowLengths=mysql_fetch_lengths(res))==NULL)
			{
				_ftprintf(stream, _T("row length error %d\n"), mysql_error(&mysql));  
				queryError = true;
			}
		}
	
		if (queryError)
		{
			if (!recoverFromDB())
			{
				_ftprintf(stream, _T("%d attempts failed to recover from DB error connection\n"),
					++tryCount);	
				Sleep(1000);
			}
			else
			{
				queryError = false;
			}			
		} 
		else
		{
			break;
		}
	}
	while (tryCount < MaxTryCount) ;

	if (queryError)
	{
		isConnected = false;
		return NULL;
	}

	ptr=row[0];
	length=rowLengths[0];
	return res;
}

void HttpServerWorker::addSignature(_TCHAR* picture_name)
{
	char buffer[256];
	int selectLength;
	MYSQL_ROW row;
	MYSQL_RES* res;
	unsigned long * lengths;
	unsigned char digest[16];
	const char* selectStatement="select picture_data from mypicture where picture_name=\"";
	sprintf(buffer, "%s", selectStatement);
	selectLength=strlen(selectStatement);
	_stprintf((_TCHAR*)(buffer+selectLength), _T("%s"), picture_name);
	selectLength=strlen(buffer);
	buffer[selectLength]='"';
	buffer[selectLength+1]='\0';

	if (mysql_real_query(&mysql, buffer, selectLength+1))
	{
		_ftprintf(stream, _T("query error %d picture_name=%s\n"), mysql_error(&mysql),
			picture_name);			
	}

	if ((res=mysql_store_result(&mysql))==NULL)
	{
		_ftprintf(stream, _T("store result error %d picture_name=%s\n"), mysql_error(&mysql),
			picture_name);
		return;
	}

	row=mysql_fetch_row(res);
	lengths=mysql_fetch_lengths(res);

	calculateMD5((unsigned char*)row[0], lengths[0], digest);
	for (int i=0; i<16; i++)
	{
		printf("%X", digest[i]);
	}
	mysql_free_result(res);
}

void* HttpServerWorker::selectPictureByIndex(int id, char*& ptr, int& length)
{
	char buffer[256];
	MYSQL_ROW row;
	unsigned long * rowLengths;
	MYSQL_RES* res;
	sprintf(buffer, "select picture_data from mypicture where id=%d;", id);
	if (mysql_real_query(&mysql, buffer, strlen(buffer)))
	{
		_ftprintf(stream, _T("query error %d\n"), mysql_error(&mysql));	
		return NULL;
	}

	if ((res=mysql_store_result(&mysql))==NULL)
	{
		_ftprintf(stream, _T("store result error %d\n"), mysql_error(&mysql));
		return NULL;
	}
	if ((row=mysql_fetch_row(res))==NULL)
	{
		_ftprintf(stream, _T("row error %d\n"), mysql_error(&mysql));

		return NULL;
	}

	if ((rowLengths=mysql_fetch_lengths(res))==NULL)
	{
		_ftprintf(stream, _T("row length error %d\n"), mysql_error(&mysql));

		return NULL;
	}
	ptr=row[0];
	length=rowLengths[0];
	return res;
}


int HttpServerWorker::selectBatchPicture(int idStart, int number, char*results[], int lengths[])
{
	int i;
	const char* selectStatement="select picture_data from mypicture where id";
	char buffer[256];
	MYSQL_ROW row;
	MYSQL_RES* res;
	unsigned long * rowLengths;
	sprintf(buffer, "%s between %d and %d;", selectStatement, idStart, idStart+number);
	if (mysql_real_query(&mysql, buffer, strlen(buffer)))
	{
		_ftprintf(stream, _T("query error %d\n"), mysql_error(&mysql));	
		return -1;
	}

	if ((res=mysql_store_result(&mysql))==NULL)
	{
		_ftprintf(stream, _T("store result error %d\n"), mysql_error(&mysql));
		return -1;
	}
	for (i=0; i<number; i++)
	{
		if ((row=mysql_fetch_row(res))==NULL)
		{
			_ftprintf(stream, _T("row  error %d\n"), mysql_error(&mysql));

			return i-1;
		}


		if ((rowLengths=mysql_fetch_lengths(res))==NULL)
		{
			_ftprintf(stream, _T("row length error %d\n"), mysql_error(&mysql));

			return i-1;
		}
		results[i]=row[0];
		lengths[i]=rowLengths[0];
	}
	return i;
}

bool HttpServerWorker::updateMediaDataRange(int mediaType)
{
	char buffer[256];
	MYSQL_ROW row;
	unsigned long * rowLengths;
	MYSQL_RES* res;
	int count;

	sprintf(buffer, "select min(id), max(id), count(id) from %s ;", mediaType==0?"mypicture":"mymusic");

	if (mysql_real_query(&mysql, buffer, strlen(buffer)))
	{
		_ftprintf(stream, _T("query error %d\n"), mysql_error(&mysql));	
		return false;
	}

	if ((res=mysql_store_result(&mysql))==NULL)
	{
		_ftprintf(stream, _T("store result error %d\n"), mysql_error(&mysql));
		return false;
	}

	if ((row=mysql_fetch_row(res))==NULL)
	{
		_ftprintf(stream, _T("row error %d\n"), mysql_error(&mysql));

		return false;
	}

	if ((rowLengths=mysql_fetch_lengths(res))==NULL)
	{
		_ftprintf(stream, _T("row length error %d\n"), mysql_error(&mysql));

		return false;
	}
	sscanf( row[0], "%d", &minMediaId[mediaType]);
	sscanf( row[1], "%d", &maxMediaId[mediaType]);
	sscanf( row[2], "%d", &count);

	maxMediaPage[mediaType] = (count + MediaBatchNumber - 1) / MediaBatchNumber;
	freeResult(res);
	return true;
}

